<?php

/* A plugin to use WP-reCAPTCHA in Contact Form 7.
 * This class needs WP-reCAPTCHA and Contact Form 7 plugin installed and activated.
 * http://wordpress.org/extend/plugins/wp-recaptcha/
 * http://wordpress.org/extend/plugins/contact-form-7/
 */


require_once('WPASDPlugin.class.php');

	//add_filter( 'cf7recaptcha_validate_recaptcha', 'cf7recaptcha_validation_filter', 10, 2 );


if (!class_exists('CF7reCAPTCHA')) {

    class CF7reCAPTCHA extends WPASDPlugin {
    
	const recaptcha_options_name = 'recaptcha_options';
	
    
	// member variables
	private $is_useable;
	
	
	// php4 Constructor
	function CF7reCAPTCHA($options_name) {
	    
	    $args = func_get_args();
	    call_user_func_array(array(&$this, "__construct"), $args);
	}
	
	
	// php5 Constructor
	function __construct($options_name) {
	    parent::__construct($options_name);
	    
	}
	
	function getClassFile() {
	    return __FILE__;
	}
	
	
	function pre_init() {
	    
	    // require the libraries
	    $this->require_library();
	    
	}
	
	
	function post_init() {
	
	    // register CF7 hooks
	    $this->register_cf7();
	    
	}
	
	// set the default options
	function register_default_options() {
	    if (is_array($this->options) && isset($this->options['reset_on_activate']) && $this->options['reset_on_activate'] !== 'on')
		return;	
		
	    $default_options = array();
	    
	    // reset on aktivate
	    $default_options['reset_on_activate'] = 'on';
	    
	    // one of {'comments_theme', 'registration_theme', 'cf7recapext_theme'}
	    $default_options['theme_selection'] = 'comments_theme';
	    
	    // one of {'red', 'white', 'blackglass', 'clean'}
	    $default_options['cf7recapext_theme'] = 'red';
	    
	    // one of {'wprecaptcha', 'cf7recapext'}
	    $default_options['language_selection'] = 'wprecaptcha';
	    
	    // one of {'en', 'nl', 'fr', 'de', 'pt', 'ru', 'es', 'tr' }
	    $default_options['cf7recapext_language'] = 'en';
	    
	    
	    // add the options based on the environment
	    WPASDPlugin::update_options($this->options_name, $default_options);
	}
	
	
	function add_settings() {
	    
	    // Theme Options Section
	    add_settings_section('cf7recapext_theme_section', __('Theme Options', 'cf7recaptcha'), array(&$this, 'echo_theme_section_info'), $this->options_name . '_page');
    	    add_settings_field('cf7recapext_theme_preselection', __('Theme Preselection', 'cf7recaptcha'), array(&$this, 'echo_theme_selection_radio'), $this->options_name . '_page', 'cf7recapext_theme_section');
    	    add_settings_field('cf7recapext_own_theme', __('Own Theme (<i>if selected</i>)', 'cf7recaptcha'), array(&$this, 'echo_theme_dropdown'), $this->options_name . '_page', 'cf7recapext_theme_section');
    	    
    	    
    	    // General Options Section
    	    add_settings_section('cf7recapext_general_section', __('General Options', 'cf7recaptcha'), array(&$this, 'echo_general_section_info'), $this->options_name . '_page');
    	    add_settings_field('cf7recapext_language_preselection', __('Language Preselection', 'cf7recaptcha'), array(&$this, 'echo_language_selection_radio'), $this->options_name . '_page', 'cf7recapext_general_section');
    	    add_settings_field('cf7recapext_own_language', __('Own Language (<i>if selected</i>)', 'cf7recaptcha'), array(&$this, 'echo_language_dropdown'), $this->options_name . '_page', 'cf7recapext_general_section');
    	    
	    // Debug Settings Section
	    add_settings_section('cf7recapext_debug_section', __('DEBUG Options', 'cf7recaptcha'), array(&$this, 'echo_debug_section_info'), $this->options_name . '_page');
	    add_settings_field('cf7recapext_reset_on_activate', __('Reset on Activate', 'cf7recaptcha'), array(&$this, 'echo_reset_on_activate_option'), $this->options_name . '_page', 'cf7recapext_debug_section');
	}
	
	function echo_theme_section_info() {
	    echo '<p>' . __('Here you can set which options to use for the themes option of the WP-reCAPTCHA forms in the Contact Form 7 forms.', 'cf7recaptcha') . "</p>\n";
	}
	
	function echo_general_section_info() {
	    echo '<p>' . __('Here you can do the same with some of the general options of WP-reCAPTCHA.', 'cf7recaptcha') . "</p>\n";
	}
	
	function echo_debug_section_info() {
	    echo '<p>' . __('Some debug options.', 'cf7recaptcha') . "</p>\n";
	}
	
	function echo_reset_on_activate_option() {
	    $checked = ($this->options['reset_on_activate'] === 'on') ? ' checked="checked" ' : '';
	    
	    echo '<input type="checkbox" id="' . $this->options_name. '[reset_on_activate]" name="' . $this->options_name. '[reset_on_activate]" value="on"' . $checked . '/>'; 
	}
	
	function validate_options($input) {
	
	    $theme_selections = 
		array(
		    'comments_theme', 		// if the theme for the comments should be used
		    'registration_theme', 	// if the theme for the registrations should be used
		    'cf7recapext_theme');	// if an own theme should be used
		    
	    $validated['theme_selection'] = 
		$this->validate_dropdown(
		    $theme_selections, 
		    'theme_selection', 
		    $input['theme_selection']);
	
	    
	    if ($validated['theme_selection'] === 'cf7recapext_theme') {
	    
		    $themes = 
			array(
			    'red',
			    'white',
			    'blackglass',
			    'clean');
	    
		    $validated['cf7recapext_theme'] = 
			$this->validate_dropdown(
			    $themes,
			    'cf7recapext_theme',
			    $input['cf7recapext_theme']);
	    } /**else if () {
		
		$validated['cf7recapext_theme'] = 
		    $this->options['cf7recapext_theme'];
	    }*/
	    
	    
	    $language_selections = 
		array (
		    'wprecaptcha',
		    'cf7recapext'
		    );
	    
	    $validated['language_selection'] = 
		$this->validate_dropdown(
		    $language_selections,
		    'language_selection',
		    $input['language_selection']);
		    
	    if ($validated['language_selection'] === 'cf7recapext') {
	    
		$recaptcha_languages = 
		    array(
			'en', 
			'nl',
			'fr', 
			'de',
			'pt',
			'ru',
			'es',
			'tr');
	    
		$validated['cf7recapext_language'] =
		    $this->validate_dropdown(
			$recaptcha_languages,
			'cf7recapext_language',
			$input['cf7recapext_language']); 
	    }
	    
	    $validated['reset_on_activate'] = ($input['reset_on_activate'] === 'on') ? 'on' : 'off';
	    
	    return $validated;
	}
	
	function require_library() {
	    
	}
	
	function register_actions() {
	    global $wp_version;
	
	    // global configuration
	    /**
	    if ( version_compare( $wp_version, '2.7alpha', '>' ) ) {
		add_action( 'after_plugin_row_' . CF7RECAP_BASENAME, array(&$this, 'add_config_form' ), 10, 3 );		
	    } else {
	        add_action( 'after_plugin_row', array(&$this, 'add_config_form' ), 10, 3 );
	    }*/
	
	
	    add_action( 'admin_notices', array(&$this, 'admin_notice') );
	    
	    
	    if ($this->useable()) {
	
		add_action( 'admin_init', array(&$this, 'tag_generator_recaptcha'), 46 );
		//add_action( 'admin_footer', array(&$this, 'add_script2admin_footer') );
	    
	    }
	
	}
	
	
	function register_filters() {
	
	    /**
	    if ( version_compare( $wp_version, '2.7alpha', '>' ) ) {
		add_filter( 'plugin_action_links_' . CF7RECAP_BASENAME, array(&$this, 'add_settings_link'), 10, 2 );
		
	    } else {
	        add_filter( 'plugin_action_links', array(&$this, 'add_settings_link'), 10, 2 );
	    }*/
	
	    if ( $this->useable() ) {
	    
		add_filter( 'wpcf7_validate_recaptcha', array(&$this, 'recaptcha_validation_filter'), 10, 2 );
		
		add_filter( 'wpcf7_ajax_json_echo', array(&$this, 'ajax_json_echo_filter') );
		
	    }

		
	}
	
	function register_cf7() {
	
	    // CF7 Shortcode Handler
	    if (function_exists('wpcf7_add_shortcode') && $this->useable() ) {
	    
		wpcf7_add_shortcode( 'recaptcha', array(&$this, 'shortcode_handler'), true );	
		
	    }
	    
		
	}
	
	
	function useable() {
	    if (!isset($this->is_useable)) {
		$this->is_useable = $this->is_recaptcha_active() && $this->is_cf7_active();
	    }
		
	    return $this->is_useable;
	}
    
	function is_recaptcha_active() {
	
	    return in_array( 
		'wp-recaptcha/wp-recaptcha.php', 
		apply_filters( 
		    'active_plugins', 
		    get_option( 
			'active_plugins' ) ) );
	
	}
	
	function is_cf7_active() {
	    return in_array( 
		'contact-form-7/wp-contact-form-7.php', 
		apply_filters( 
		    'active_plugins', 
		    get_option( 
			'active_plugins' ) ) );
	}
	
	/**
	function add_settings_link( $links, $file ) {
	    
	    array_unshift(
		$links,
		sprintf( '<a id="cf7recap-pluginconflink" href="javascript:void(0)" title="Configure this plugin">%s</a>', __('Settings') )
		);
		
	    return $links;
	}*/
	
	/**
	function add_script2admin_footer() {
	    
	    ?>
	    <script type="text/javascript">
		jQuery(document).ready(
	    </script>
	    <?php
	
	
	}*/
	
	/**
	 * 
	 * @return
	 * @param $wm_pluginfile Object
	 * @param $wm_plugindata Object (array)
	 * @param $wm_context    Object (all, active, inactive)
	 */
	 /**
	function add_config_form($cf7_pluginfile, $cf7_plugindata, $cf7_context) {

	    $config_visibility = 'class="';
	    if ( !isset($_GET['show']) || 'cf7config' != $_GET['show']) {
	     
		$config_visibility .= 'config_hidden' ;
	    }
	    $config_visibility .= '"';
	    
	    ?>
	    <tr id="cf7_config_tr">
		<td colspan="3">
		
		    <div id="cf7_config_row" <?php echo $config_visibility; ?>>
			<p> test </p>
		    </div>		
		</td>
	    </tr>
	    <?php 
	    
	}*/
	
	
	function show_settings_page() {
	    include('settings.php');
	}
	
	function echo_theme_selection_radio() {
	
	    $recaptcha_options = WPASDPlugin::retrieve_options(CF7reCAPTCHA::recaptcha_options_name);
	    
	    $themes =
		array (
		    'red'        => __('Red',         'recaptcha'),
		    'white'      => __('White',       'recaptcha'),
		    'blackglass' => __('Black Glass', 'recaptcha'),
		    'clean'      => __('Clean',       'recaptcha'));
	    
	    $comments_theme = (is_array($recaptcha_options) && isset($recaptcha_options['comments_theme']) ) ? ' (currently: <i>' . $themes[$recaptcha_options['comments_theme']] . '</i>)' : '';
	    $registration_theme = (is_array($recaptcha_options) && isset($recaptcha_options['registration_theme']) ) ? ' (currently: <i>' . $themes[$recaptcha_options['registration_theme']] . '</i>)' : '';
	
	    $theme_options = 
		array (
		    __('WP-reCAPTCHA Comments Theme',     'cf7recaptcha') . $comments_theme     => 'comments_theme', 		
		    __('WP-reCAPTCHA Registration Theme', 'cf7recaptcha') . $registration_theme => 'registration_theme', 	
		    __('Own Theme (<i>select below</i>)', 'cf7recaptcha')                       => 'cf7recapext_theme'
		    );
	
	    $this->echo_radios($this->options_name . '[theme_selection]', $theme_options, $this->options['theme_selection']);
	}
	
	function echo_theme_dropdown() {
	    $themes =
		array (
		    __('Red',         'recaptcha') => 'red',
		    __('White',       'recaptcha') => 'white',
		    __('Black Glass', 'recaptcha') => 'blackglass',
		    __('Clean',       'recaptcha') => 'clean');
		
	    echo '<label for="' . $this->options_name . '[cf7recapext_theme]">' . __('Theme:', 'cf7recaptcha') . "</label>\n";     
	    $this->echo_dropdown($this->options_name . '[cf7recapext_theme]', $themes, $this->options['cf7recapext_theme']);
	}
	
	function echo_language_selection_radio() {
	
	    $recaptcha_options = WPASDPlugin::retrieve_options(CF7reCAPTCHA::recaptcha_options_name);
	    
	    $languages =
		array (
		    'en' => __('English',    'recaptcha'),
		    'nl' => __('Dutch',      'recaptcha'),
		    'fr' => __('French',     'recaptcha'),
		    'de' => __('German',     'recaptcha'),
		    'pt' => __('Portuguese', 'recaptcha'),
		    'ru' => __('Russian',    'recaptcha'),
		    'es' => __('Spanish',    'recaptcha'),
		    'tr' => __('Turkish',    'recaptcha'));
	    
	    $recaptcha_language = (is_array($recaptcha_options) && isset($recaptcha_options['recaptcha_language']) ) ? ' (currently: <i>' . $languages[$recaptcha_options['recaptcha_language']] . '</i>)' : '';
		    
	    $language_options = 
		array (
		    __('WP-reCAPTCHA Language',              'cf7recaptcha') . $recaptcha_language => 'wprecaptcha',
		    __('Own Language (<i>select below</i>)', 'cf7recaptcha')                       => 'cf7recapext'
		    );
	
	    $this->echo_radios($this->options_name . '[language_selection]', $language_options, $this->options['language_selection']);
	}
	
	function echo_language_dropdown() {
	    $languages = 
		array(
		    __('English',    'recaptcha') => 'en',
		    __('Dutch',      'recaptcha') => 'nl',
		    __('French',     'recaptcha') => 'fr',
		    __('German',     'recaptcha') => 'de',
		    __('Portuguese', 'recaptcha') => 'pt',
		    __('Russian',    'recaptcha') => 'ru',
		    __('Spanish',    'recaptcha') => 'es',
		    __('Turkish',    'recaptcha') => 'tr'
		    );
	    
	    echo '<label for="' . $this->options_name . '[cf7recapext_language]">' . __('Language:', 'cf7recaptcha') . "</label>\n";
	    $this->echo_dropdown($this->options_name . '[cf7recapext_language]', $languages, $this->options['cf7recapext_language']);
	}
	
	function ajax_json_echo_filter( $items ) {
	    if ( ! is_array( $items['onSubmit'] ) )
		$items['onSubmit'] = array();

	    $items['onSubmit'][] = 'Recaptcha.reload();';

	    return $items;
	}


	function recaptcha_validation_filter( $result, $tag ) {
	    global $recaptcha;
	    
	    $name = $tag['name'];
	    
	    if(!$this->is_multi_blog()) {
		
		$errors = new WP_Error();
		
		$recaptcha->validate_recaptcha_response($errors);
		
		$error_list = $errors->get_error_messages(null);
		
		if (!empty($error_list)) {
		
		    $result['valid'] = false;
		    
		    $error_out = "";
		    
		    foreach ($error_list as $value) {
		    
			$error_out .= $value;	
			
		    }
		    
		    $result['reason'][$name] = $error_out;
		}
		
	    } else {
	    
	        $recaptcha->validate_recaptcha_response_wpmu($result);
		
	    }
	

	    return $result;
	}
	
	
	function tag_generator_recaptcha() {
	
	    if (function_exists('wpcf7_add_tag_generator') && $this->useable()) {
		wpcf7_add_tag_generator(
		    'recaptcha',
		    'reCAPTCHA',
		    'cf7recaptcha-tg-pane',
		    array(&$this, 'tag_pane'));
	    }
	    
	}

	
	function shortcode_handler( $tag ) {
	    global $wpcf7_contact_form, $recaptcha;

	    $name = $tag['name'];
	    
	    $recaptcha_options = WPASDPlugin::retrieve_options(CF7reCAPTCHA::recaptcha_options_name);
	    
	    
	    $used_theme = '';
	    
	    if ($this->options['theme_selection'] === 'comments_theme' && isset($recaptcha_options['comments_theme'])) {
		$used_theme = $recaptcha_options['comments_theme'];
	    } else if ($this->options['theme_selection'] === 'registration_theme' && isset($recaptcha_options['registration_theme'])) {
		$used_theme = $recaptcha_options['registration_theme'];
	    } else if ($this->options['theme_selection'] === 'cf7recapext_theme' && isset($this->options['cf7recapext_theme'])) {
		$used_theme = $this->options['cf7recapext_theme'];
	    } else {
		$used_theme = 'red';
	    }
	    
	    $used_language = '';
	    
	    if ($this->options['language_selection'] === 'wprecaptcha' && isset($recaptcha_options['recaptcha_language'])) {
		$used_language = $recaptcha_options['recaptcha_language'];
	    } else if ($this->options['language_selection'] === 'cf7recapext' && isset($this->options['cf7recapext_language'])) {
		$used_language = $this->options['cf7recapext_language'];
	    } else {
		$used_language = 'en';
	    }
	    
	    $js_options = <<<OPTS
	    <script type='text/javascript'>
		var RecaptchaOptions = { theme : '{$used_theme}', lang : '{$used_language}'};
	    </script>
OPTS;

	    $use_ssl = isset( $_SERVER['HTTPS'] ) && $_SERVER['HTTPS'] == 'on';


	    $html = $js_options . $recaptcha->get_recaptcha_html( null, $use_ssl );

	    $validation_error = '';
	    if ( is_a( $wpcf7_contact_form, 'WPCF7_ContactForm' ) )
		$validation_error = $wpcf7_contact_form->validation_error( $name );
	
	    //$html .= '<div id="recaptcha-submit-btn-area"></div>';
	    $html .= '<span class="wpcf7-form-control-wrap ' . $name . '">' . $validation_error . '</span>';

	    return $html;
	}


	function tag_pane( &$contact_form ) {
?>
<div id="cf7recaptcha-tg-pane" class="hidden">
<form action="">
<table>

<?php if ( ! $this->useable() ) : ?>
<tr><td colspan="2"><strong style="color: #e6255b">you need reCAPTCHA</strong><br /></td></tr>
<?php endif; ?>

<tr><td><?php echo esc_html( __( 'Name', 'cf7recaptcha' ) ); ?><br /><input type="text" name="name" class="tg-name oneline" /></td><td></td></tr>
</table>

<div class="tg-tag"><?php echo esc_html( __( "Copy this code and paste it into the form left.", 'cf7recaptcha' ) ); ?>
<br />
<input type="text" name="recaptcha" class="tag" readonly="readonly" onfocus="this.select()" />
</div>
</form>
</div>
<?php
	}	
	
	
	function admin_notice() {
	    global $plugin_page;

    	    //if ( ! isset( $plugin_page ) || 0 !== strpos( $plugin_page, WPCF7_PLUGIN_NAME ) )
	    //	return;

	    if ( ! $this->is_cf7_active() ) :

?>
<div id="message" class="updated fade"><p>
<?php _e( "You are using Contact Form 7 reCAPTCHA Extension." , 'cf7recaptcha'); ?> 
<?php _e( "This works with the Contact Form 7 plugin, but the Contact Form 7 plugin is not activated.", 'cf7recaptcha' ); ?>
 &mdash; Contact Form 7 <a href="http://wordpress.org/extend/plugins/contact-form-7/">http://wordpress.org/extend/plugins/contact-form-7/</a><p>
</div>
<?php
	    endif;


	    if ( ! $this->is_recaptcha_active() ) :

?>
<div id="message" class="updated fade"><p>
<?php _e( "You are using Contact Form 7 reCAPTCHA Extension." , 'cf7recaptcha'); ?> 
<?php _e( "This works with the WP-reCAPTCHA plugin, but the WP-reCAPTCHA plugin is not activated.", 'cf7recaptcha' ); ?>
 &mdash; WP-reCAPTCHA <a href="http://wordpress.org/extend/plugins/wp-recaptcha/">http://wordpress.org/extend/plugins/wp-recaptcha/</a><p>
</div>
<?php
	    endif;


	    
	    
	}
    
    } // end of class declaration

} // end of class exists clause

?>